@@ -47,6 +47,7 @@ use tokio::io::AsyncWrite;
 use tracing::info;
 use url::Url;
 
+pub use crate::aws::checksum::Checksum;
 use crate::aws::client::{S3Client, S3Config};
 use crate::aws::credential::{
     AwsCredential, CredentialProvider, InstanceCredentialProvider,
@@ -59,6 +60,7 @@ use crate::{
     Result, RetryConfig, StreamExt,
 };
 
+mod checksum;
 mod client;
 mod credential;
 
@@ -101,6 +103,9 @@ enum Error {
         source: std::num::ParseIntError,
     },
 
+    #[snafu(display("Invalid Checksum algorithm"))]
+    InvalidChecksumAlgorithm,
+
     #[snafu(display("Missing region"))]
     MissingRegion,
 
@@ -268,7 +273,22 @@ impl ObjectStore for AmazonS3 {
     ) -> Result<BoxStream<'_, Result<ObjectMeta>>> {
         let stream = self
             .client
-            .list_paginated(prefix, false)
+            .list_paginated(prefix, false, None)
+            .map_ok(|r| futures::stream::iter(r.objects.into_iter().map(Ok)))
+            .try_flatten()
+            .boxed();
+
+        Ok(stream)
+    }
+
+    async fn list_with_offset(
+        &self,
+        prefix: Option<&Path>,
+        offset: &Path,
+    ) -> Result<BoxStream<'_, Result<ObjectMeta>>> {
+        let stream = self
+            .client
+            .list_paginated(prefix, false, Some(offset))
             .map_ok(|r| futures::stream::iter(r.objects.into_iter().map(Ok)))
             .try_flatten()
             .boxed();
@@ -277,7 +297,7 @@ impl ObjectStore for AmazonS3 {
     }
 
     async fn list_with_delimiter(&self, prefix: Option<&Path>) -> Result<ListResult> {
-        let mut stream = self.client.list_paginated(prefix, true);
+        let mut stream = self.client.list_paginated(prefix, true, None);
 
         let mut common_prefixes = BTreeSet::new();
         let mut objects = Vec::new();
@@ -385,6 +405,8 @@ pub struct AmazonS3Builder {
     retry_config: RetryConfig,
     imdsv1_fallback: bool,
     virtual_hosted_style_request: bool,
+    unsigned_payload: bool,
+    checksum_algorithm: Option<Checksum>,
     metadata_endpoint: Option<String>,
     profile: Option<String>,
     client_options: ClientOptions,
@@ -407,7 +429,7 @@ pub struct AmazonS3Builder {
 /// let typed_options = vec![
 ///     (AmazonS3ConfigKey::DefaultRegion, "my-default-region"),
 /// ];
-/// let azure = AmazonS3Builder::new()
+/// let aws = AmazonS3Builder::new()
 ///     .try_with_options(options)
 ///     .unwrap()
 ///     .try_with_options(typed_options)
@@ -504,6 +526,20 @@ pub enum AmazonS3ConfigKey {
     /// - `virtual_hosted_style_request`
     VirtualHostedStyleRequest,
 
+    /// Avoid computing payload checksum when calculating signature.
+    ///
+    /// See [`AmazonS3Builder::with_unsigned_payload`] for details.
+    ///
+    /// Supported keys:
+    /// - `aws_unsigned_payload`
+    /// - `unsigned_payload`
+    UnsignedPayload,
+
+    /// Set the checksum algorithm for this client
+    ///
+    /// See [`AmazonS3Builder::with_checksum_algorithm`]
+    Checksum,
+
     /// Set the instance metadata endpoint
     ///
     /// See [`AmazonS3Builder::with_metadata_endpoint`] for details.
@@ -535,6 +571,8 @@ impl AsRef<str> for AmazonS3ConfigKey {
             Self::DefaultRegion => "aws_default_region",
             Self::MetadataEndpoint => "aws_metadata_endpoint",
             Self::Profile => "aws_profile",
+            Self::UnsignedPayload => "aws_unsigned_payload",
+            Self::Checksum => "aws_checksum_algorithm",
         }
     }
 }
@@ -563,6 +601,8 @@ impl FromStr for AmazonS3ConfigKey {
             "aws_profile" | "profile" => Ok(Self::Profile),
             "aws_imdsv1_fallback" | "imdsv1_fallback" => Ok(Self::ImdsV1Fallback),
             "aws_metadata_endpoint" | "metadata_endpoint" => Ok(Self::MetadataEndpoint),
+            "aws_unsigned_payload" | "unsigned_payload" => Ok(Self::UnsignedPayload),
+            "aws_checksum_algorithm" | "checksum_algorithm" => Ok(Self::Checksum),
             _ => Err(Error::UnknownConfigurationKey { key: s.into() }.into()),
         }
     }
@@ -679,6 +719,14 @@ impl AmazonS3Builder {
                 self.metadata_endpoint = Some(value.into())
             }
             AmazonS3ConfigKey::Profile => self.profile = Some(value.into()),
+            AmazonS3ConfigKey::UnsignedPayload => {
+                self.unsigned_payload = str_is_truthy(&value.into())
+            }
+            AmazonS3ConfigKey::Checksum => {
+                let algorithm = Checksum::try_from(&value.into())
+                    .map_err(|_| Error::InvalidChecksumAlgorithm)?;
+                self.checksum_algorithm = Some(algorithm)
+            }
         };
         Ok(self)
     }
@@ -705,13 +753,9 @@ impl AmazonS3Builder {
     fn parse_url(&mut self, url: &str) -> Result<()> {
         let parsed = Url::parse(url).context(UnableToParseUrlSnafu { url })?;
         let host = parsed.host_str().context(UrlNotRecognisedSnafu { url })?;
-        let validate = |s: &str| match s.contains('.') {
-            true => Err(UrlNotRecognisedSnafu { url }.build()),
-            false => Ok(s.to_string()),
-        };
 
         match parsed.scheme() {
-            "s3" | "s3a" => self.bucket_name = Some(validate(host)?),
+            "s3" | "s3a" => self.bucket_name = Some(host.to_string()),
             "https" => match host.splitn(4, '.').collect_tuple() {
                 Some(("s3", bucket, "amazonaws", "com")) => {
                     self.bucket_name = Some(bucket.to_string());
@@ -822,6 +866,23 @@ impl AmazonS3Builder {
         self
     }
 
+    /// Sets if unsigned payload option has to be used.
+    /// See [unsigned payload option](https://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-header-based-auth.html)
+    /// * false (default): Signed payload option is used, where the checksum for the request body is computed and included when constructing a canonical request.
+    /// * true: Unsigned payload option is used. `UNSIGNED-PAYLOAD` literal is included when constructing a canonical request,
+    pub fn with_unsigned_payload(mut self, unsigned_payload: bool) -> Self {
+        self.unsigned_payload = unsigned_payload;
+        self
+    }
+
+    /// Sets the [checksum algorithm] which has to be used for object integrity check during upload.
+    ///
+    /// [checksum algorithm]: https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html
+    pub fn with_checksum_algorithm(mut self, checksum_algorithm: Checksum) -> Self {
+        self.checksum_algorithm = Some(checksum_algorithm);
+        self
+    }
+
     /// Set the [instance metadata endpoint](https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html),
     /// used primarily within AWS EC2.
     ///
@@ -967,6 +1028,8 @@ impl AmazonS3Builder {
             credentials,
             retry_config: self.retry_config,
             client_options: self.client_options,
+            sign_payload: !self.unsigned_payload,
+            checksum: self.checksum_algorithm,
         };
 
         let client = Arc::new(S3Client::new(config)?);
@@ -1125,6 +1188,8 @@ mod tests {
             "AWS_CONTAINER_CREDENTIALS_RELATIVE_URI",
             &container_creds_relative_uri,
         );
+        env::set_var("AWS_UNSIGNED_PAYLOAD", "true");
+        env::set_var("AWS_CHECKSUM_ALGORITHM", "sha256");
 
         let builder = AmazonS3Builder::from_env();
         assert_eq!(builder.access_key_id.unwrap(), aws_access_key_id.as_str());
@@ -1136,9 +1201,10 @@ mod tests {
 
         assert_eq!(builder.endpoint.unwrap(), aws_endpoint);
         assert_eq!(builder.token.unwrap(), aws_session_token);
-
         let metadata_uri = format!("{METADATA_ENDPOINT}{container_creds_relative_uri}");
         assert_eq!(builder.metadata_endpoint.unwrap(), metadata_uri);
+        assert_eq!(builder.checksum_algorithm.unwrap(), Checksum::SHA256);
+        assert!(builder.unsigned_payload);
     }
 
     #[test]
@@ -1154,6 +1220,8 @@ mod tests {
             ("aws_default_region", aws_default_region.clone()),
             ("aws_endpoint", aws_endpoint.clone()),
             ("aws_session_token", aws_session_token.clone()),
+            ("aws_unsigned_payload", "true".to_string()),
+            ("aws_checksum_algorithm", "sha256".to_string()),
         ]);
 
         let builder = AmazonS3Builder::new()
@@ -1166,6 +1234,8 @@ mod tests {
         assert_eq!(builder.region.unwrap(), aws_default_region);
         assert_eq!(builder.endpoint.unwrap(), aws_endpoint);
         assert_eq!(builder.token.unwrap(), aws_session_token);
+        assert_eq!(builder.checksum_algorithm.unwrap(), Checksum::SHA256);
+        assert!(builder.unsigned_payload);
     }
 
     #[test]
@@ -1181,6 +1251,7 @@ mod tests {
             (AmazonS3ConfigKey::DefaultRegion, aws_default_region.clone()),
             (AmazonS3ConfigKey::Endpoint, aws_endpoint.clone()),
             (AmazonS3ConfigKey::Token, aws_session_token.clone()),
+            (AmazonS3ConfigKey::UnsignedPayload, "true".to_string()),
         ]);
 
         let builder = AmazonS3Builder::new()
@@ -1193,6 +1264,7 @@ mod tests {
         assert_eq!(builder.region.unwrap(), aws_default_region);
         assert_eq!(builder.endpoint.unwrap(), aws_endpoint);
         assert_eq!(builder.token.unwrap(), aws_session_token);
+        assert!(builder.unsigned_payload);
     }
 
     #[test]
@@ -1220,6 +1292,18 @@ mod tests {
         list_with_delimiter(&integration).await;
         rename_and_copy(&integration).await;
         stream_get(&integration).await;
+
+        // run integration test with unsigned payload enabled
+        let config = maybe_skip_integration!().with_unsigned_payload(true);
+        let is_local = matches!(&config.endpoint, Some(e) if e.starts_with("http://"));
+        let integration = config.build().unwrap();
+        put_get_delete_list_opts(&integration, is_local).await;
+
+        // run integration test with checksum set to sha256
+        let config = maybe_skip_integration!().with_checksum_algorithm(Checksum::SHA256);
+        let is_local = matches!(&config.endpoint, Some(e) if e.starts_with("http://"));
+        let integration = config.build().unwrap();
+        put_get_delete_list_opts(&integration, is_local).await;
     }
 
     #[tokio::test]
@@ -1316,6 +1400,15 @@ mod tests {
         builder.parse_url("s3://bucket/path").unwrap();
         assert_eq!(builder.bucket_name, Some("bucket".to_string()));
 
+        let mut builder = AmazonS3Builder::new();
+        builder
+            .parse_url("s3://buckets.can.have.dots/path")
+            .unwrap();
+        assert_eq!(
+            builder.bucket_name,
+            Some("buckets.can.have.dots".to_string())
+        );
+
         let mut builder = AmazonS3Builder::new();
         builder
             .parse_url("https://s3.bucket.amazonaws.com")
@@ -1332,7 +1425,6 @@ mod tests {
 
         let err_cases = [
             "mailto://bucket/path",
-            "s3://bucket.mydomain/path",
             "https://s3.bucket.mydomain.com",
             "https://s3.bucket.foo.amazonaws.com",
             "https://bucket.mydomain.region.amazonaws.com",
