@@ -27,8 +27,8 @@ use futures::future::BoxFuture;
 use futures::FutureExt;
 use futures::{stream::BoxStream, StreamExt};
 use snafu::{ensure, OptionExt, ResultExt, Snafu};
-use std::fs::{metadata, symlink_metadata, File};
-use std::io::{Read, Seek, SeekFrom, Write};
+use std::fs::{metadata, symlink_metadata, File, OpenOptions};
+use std::io::{ErrorKind, Read, Seek, SeekFrom, Write};
 use std::ops::Range;
 use std::pin::Pin;
 use std::sync::Arc;
@@ -65,6 +65,11 @@ pub(crate) enum Error {
         source: io::Error,
     },
 
+    #[snafu(display("Unable to rename file: {}", source))]
+    UnableToRenameFile {
+        source: io::Error,
+    },
+
     #[snafu(display("Unable to create dir {}: {}", path.display(), source))]
     UnableToCreateDir {
         source: io::Error,
@@ -264,13 +269,15 @@ impl Config {
 impl ObjectStore for LocalFileSystem {
     async fn put(&self, location: &Path, bytes: Bytes) -> Result<()> {
         let path = self.config.path_to_filesystem(location)?;
-
         maybe_spawn_blocking(move || {
-            let mut file = open_writable_file(&path)?;
+            let (mut file, suffix) = new_staged_upload(&path)?;
+            let staging_path = staged_upload_path(&path, &suffix);
 
             file.write_all(&bytes)
                 .context(UnableToCopyDataToFileSnafu)?;
 
+            std::fs::rename(staging_path, path).context(UnableToRenameFileSnafu)?;
+
             Ok(())
         })
         .await
@@ -282,28 +289,10 @@ impl ObjectStore for LocalFileSystem {
     ) -> Result<(MultipartId, Box<dyn AsyncWrite + Unpin + Send>)> {
         let dest = self.config.path_to_filesystem(location)?;
 
-        // Generate an id in case of concurrent writes
-        let mut multipart_id = 1;
-
-        // Will write to a temporary path
-        let staging_path = loop {
-            let staging_path = get_upload_stage_path(&dest, &multipart_id.to_string());
-
-            match std::fs::metadata(&staging_path) {
-                Err(err) if err.kind() == io::ErrorKind::NotFound => break staging_path,
-                Err(err) => {
-                    return Err(Error::UnableToCopyDataToFile { source: err }.into())
-                }
-                Ok(_) => multipart_id += 1,
-            }
-        };
-        let multipart_id = multipart_id.to_string();
-
-        let file = open_writable_file(&staging_path)?;
-
+        let (file, suffix) = new_staged_upload(&dest)?;
         Ok((
-            multipart_id.clone(),
-            Box::new(LocalUpload::new(dest, multipart_id, Arc::new(file))),
+            suffix.clone(),
+            Box::new(LocalUpload::new(dest, suffix, Arc::new(file))),
         ))
     }
 
@@ -313,7 +302,7 @@ impl ObjectStore for LocalFileSystem {
         multipart_id: &MultipartId,
     ) -> Result<()> {
         let dest = self.config.path_to_filesystem(location)?;
-        let staging_path: PathBuf = get_upload_stage_path(&dest, multipart_id);
+        let staging_path: PathBuf = staged_upload_path(&dest, multipart_id);
 
         maybe_spawn_blocking(move || {
             std::fs::remove_file(&staging_path)
@@ -323,6 +312,53 @@ impl ObjectStore for LocalFileSystem {
         .await
     }
 
+    async fn append(
+        &self,
+        location: &Path,
+    ) -> Result<Box<dyn AsyncWrite + Unpin + Send>> {
+        #[cfg(not(target_arch = "wasm32"))]
+        // Get the path to the file from the configuration.
+        let path = self.config.path_to_filesystem(location)?;
+        loop {
+            // Create new `OpenOptions`.
+            let mut options = tokio::fs::OpenOptions::new();
+
+            // Attempt to open the file with the given options.
+            match options
+                .truncate(false)
+                .append(true)
+                .create(true)
+                .open(&path)
+                .await
+            {
+                // If the file was successfully opened, return it wrapped in a boxed `AsyncWrite` trait object.
+                Ok(file) => return Ok(Box::new(file)),
+                // If the error is that the file was not found, attempt to create the file and any necessary parent directories.
+                Err(err) if err.kind() == ErrorKind::NotFound => {
+                    // Get the path to the parent directory of the file.
+                    let parent = path
+                        .parent()
+                        // If the parent directory does not exist, return a `UnableToCreateFileSnafu` error.
+                        .context(UnableToCreateFileSnafu { path: &path, err })?;
+
+                    // Create the parent directory and any necessary ancestors.
+                    tokio::fs::create_dir_all(parent)
+                        .await
+                        // If creating the directory fails, return a `UnableToCreateDirSnafu` error.
+                        .context(UnableToCreateDirSnafu { path: parent })?;
+                    // Try again to open the file.
+                    continue;
+                }
+                // If any other error occurs, return a `UnableToOpenFile` error.
+                Err(source) => {
+                    return Err(Error::UnableToOpenFile { source, path }.into())
+                }
+            }
+        }
+        #[cfg(target_arch = "wasm32")]
+        Err(super::Error::NotImplemented)
+    }
+
     async fn get(&self, location: &Path) -> Result<GetResult> {
         let path = self.config.path_to_filesystem(location)?;
         maybe_spawn_blocking(move || {
@@ -364,13 +400,20 @@ impl ObjectStore for LocalFileSystem {
         let location = location.clone();
 
         maybe_spawn_blocking(move || {
-            let file = open_file(&path)?;
-            let metadata =
-                file.metadata().map_err(|e| Error::UnableToAccessMetadata {
-                    source: e.into(),
-                    path: location.to_string(),
-                })?;
-
+            let metadata = match metadata(&path) {
+                Err(e) => Err(if e.kind() == ErrorKind::NotFound {
+                    Error::NotFound {
+                        path: path.clone(),
+                        source: e,
+                    }
+                } else {
+                    Error::UnableToAccessMetadata {
+                        source: e.into(),
+                        path: location.to_string(),
+                    }
+                }),
+                Ok(m) => Ok(m),
+            }?;
             convert_metadata(metadata, location)
         })
         .await
@@ -553,9 +596,40 @@ impl ObjectStore for LocalFileSystem {
     }
 }
 
-fn get_upload_stage_path(dest: &std::path::Path, multipart_id: &MultipartId) -> PathBuf {
+/// Generates a unique file path `{base}#{suffix}`, returning the opened `File` and `suffix`
+///
+/// Creates any directories if necessary
+fn new_staged_upload(base: &std::path::Path) -> Result<(File, String)> {
+    let mut multipart_id = 1;
+    loop {
+        let suffix = multipart_id.to_string();
+        let path = staged_upload_path(base, &suffix);
+        let mut options = OpenOptions::new();
+        match options.read(true).write(true).create_new(true).open(&path) {
+            Ok(f) => return Ok((f, suffix)),
+            Err(e) if e.kind() == ErrorKind::AlreadyExists => {
+                multipart_id += 1;
+            }
+            Err(err) if err.kind() == ErrorKind::NotFound => {
+                let parent = path
+                    .parent()
+                    .context(UnableToCreateFileSnafu { path: &path, err })?;
+
+                std::fs::create_dir_all(parent)
+                    .context(UnableToCreateDirSnafu { path: parent })?;
+
+                continue;
+            }
+            Err(source) => return Err(Error::UnableToOpenFile { source, path }.into()),
+        }
+    }
+}
+
+/// Returns the unique upload for the given path and suffix
+fn staged_upload_path(dest: &std::path::Path, suffix: &str) -> PathBuf {
     let mut staging_path = dest.as_os_str().to_owned();
-    staging_path.push(format!("#{multipart_id}"));
+    staging_path.push("#");
+    staging_path.push(suffix);
     staging_path.into()
 }
 
@@ -700,7 +774,7 @@ impl AsyncWrite for LocalUpload {
                         Poll::Ready(res) => {
                             res?;
                             let staging_path =
-                                get_upload_stage_path(&self.dest, &self.multipart_id);
+                                staged_upload_path(&self.dest, &self.multipart_id);
                             let dest = self.dest.clone();
                             self.inner_state = LocalUploadState::Committing(Box::pin(
                                 runtime
@@ -741,7 +815,7 @@ impl AsyncWrite for LocalUpload {
                 }
             }
         } else {
-            let staging_path = get_upload_stage_path(&self.dest, &self.multipart_id);
+            let staging_path = staged_upload_path(&self.dest, &self.multipart_id);
             match &mut self.inner_state {
                 LocalUploadState::Idle(file) => {
                     let file = Arc::clone(file);
@@ -802,33 +876,6 @@ fn open_file(path: &PathBuf) -> Result<File> {
     Ok(file)
 }
 
-fn open_writable_file(path: &PathBuf) -> Result<File> {
-    match File::create(path) {
-        Ok(f) => Ok(f),
-        Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
-            let parent = path
-                .parent()
-                .context(UnableToCreateFileSnafu { path: &path, err })?;
-            std::fs::create_dir_all(parent)
-                .context(UnableToCreateDirSnafu { path: parent })?;
-
-            match File::create(path) {
-                Ok(f) => Ok(f),
-                Err(err) => Err(Error::UnableToCreateFile {
-                    path: path.to_path_buf(),
-                    err,
-                }
-                .into()),
-            }
-        }
-        Err(err) => Err(Error::UnableToCreateFile {
-            path: path.to_path_buf(),
-            err,
-        }
-        .into()),
-    }
-}
-
 fn convert_entry(entry: DirEntry, location: Path) -> Result<ObjectMeta> {
     let metadata = entry
         .metadata()
@@ -1311,3 +1358,120 @@ mod tests {
         integration.list_with_delimiter(Some(&path)).await.unwrap();
     }
 }
+
+#[cfg(not(target_arch = "wasm32"))]
+#[cfg(test)]
+mod not_wasm_tests {
+    use crate::local::LocalFileSystem;
+    use crate::{ObjectStore, Path};
+    use bytes::Bytes;
+    use tempfile::TempDir;
+    use tokio::io::AsyncWriteExt;
+
+    #[tokio::test]
+    async fn creates_dir_if_not_present_append() {
+        let root = TempDir::new().unwrap();
+        let integration = LocalFileSystem::new_with_prefix(root.path()).unwrap();
+
+        let location = Path::from("nested/file/test_file");
+
+        let data = Bytes::from("arbitrary data");
+        let expected_data = data.clone();
+
+        let mut writer = integration.append(&location).await.unwrap();
+
+        writer.write_all(data.as_ref()).await.unwrap();
+
+        let read_data = integration
+            .get(&location)
+            .await
+            .unwrap()
+            .bytes()
+            .await
+            .unwrap();
+        assert_eq!(&*read_data, expected_data);
+    }
+
+    #[tokio::test]
+    async fn unknown_length_append() {
+        let root = TempDir::new().unwrap();
+        let integration = LocalFileSystem::new_with_prefix(root.path()).unwrap();
+
+        let location = Path::from("some_file");
+
+        let data = Bytes::from("arbitrary data");
+        let expected_data = data.clone();
+        let mut writer = integration.append(&location).await.unwrap();
+
+        writer.write_all(data.as_ref()).await.unwrap();
+
+        let read_data = integration
+            .get(&location)
+            .await
+            .unwrap()
+            .bytes()
+            .await
+            .unwrap();
+        assert_eq!(&*read_data, expected_data);
+    }
+
+    #[tokio::test]
+    async fn multiple_append() {
+        let root = TempDir::new().unwrap();
+        let integration = LocalFileSystem::new_with_prefix(root.path()).unwrap();
+
+        let location = Path::from("some_file");
+
+        let data = vec![
+            Bytes::from("arbitrary"),
+            Bytes::from("data"),
+            Bytes::from("gnz"),
+        ];
+
+        let mut writer = integration.append(&location).await.unwrap();
+        for d in &data {
+            writer.write_all(d).await.unwrap();
+        }
+
+        let mut writer = integration.append(&location).await.unwrap();
+        for d in &data {
+            writer.write_all(d).await.unwrap();
+        }
+
+        let read_data = integration
+            .get(&location)
+            .await
+            .unwrap()
+            .bytes()
+            .await
+            .unwrap();
+        let expected_data = Bytes::from("arbitrarydatagnzarbitrarydatagnz");
+        assert_eq!(&*read_data, expected_data);
+    }
+}
+
+#[cfg(target_family = "unix")]
+#[cfg(test)]
+mod unix_test {
+    use crate::local::LocalFileSystem;
+    use crate::{ObjectStore, Path};
+    use nix::sys::stat;
+    use nix::unistd;
+    use std::time::Duration;
+    use tempfile::TempDir;
+    use tokio::time::timeout;
+
+    #[tokio::test]
+    async fn test_head_fifo() {
+        let filename = "some_file";
+        let root = TempDir::new().unwrap();
+        let integration = LocalFileSystem::new_with_prefix(root.path()).unwrap();
+        unistd::mkfifo(&root.path().join(filename), stat::Mode::S_IRWXU).unwrap();
+        let location = Path::from(filename);
+        if (timeout(Duration::from_millis(10), integration.head(&location)).await)
+            .is_err()
+        {
+            panic!("Did not receive value within 10 ms");
+        }
+    }
+}
